//*****************************************************************************
//                    Laborprojekt, FH-Wedel 2005
//-----------------------------------------------------------------------------
//
// Modul:       - Dieses Modul beinhaltet smtliche USB-Funktionen
//              - bietet Fkt zur Initialiserung und Konfiguration der Endpoints 
//                und PLL (auf 48MHz)
//              - behandelt die gesamte USB-Enumeration (Austausch der 
//                Descriptoren bei der Treiber-Initialisierung)
//              - behandelt alle USB-Interrupts, wickelt die Bulk-Transfers ab
//
//-----------------------------------------------------------------------------
// Datum:       Copyright 2005, Andreas Schibilla
//*****************************************************************************
#include "at89c5131.h"
#include "ext_5131.h"
#include "compiler.h"
#include "usb.h"
#include "flash.h"

//*****************************************************************************
//--- globale Variablen
//*****************************************************************************
static t_usb_request      g_Request;                    // globales Setup-Request-Paket
static Uchar              g_Endpoint4Status = 0;        // fr usb_get/set_feature() und usb_get_status()
static Uchar              g_Endpoint5Status = 0;
static Uchar              g_ConfigurationNo = 0;        // fr usb_get/set_configuration()
static bit                g_zlp;                        // fr usb_get_descriptor()
static Uchar              *g_pbuffer;

Uchar                     g_buf[64];


//*****************************************************************************
//--- Konstanten
//*****************************************************************************

//--- fr USB DEVICE DESCRIPTOR
#define EP_CONTROL_LENGTH             32
#define MAN_INDEX                     0x01
#define PROD_INDEX                    0x02
#define SN_INDEX                      0x03
                                      
//--- Setup-Pakete                    
#define GET_STATUS                    0x00
#define CLEAR_FEATURE                 0x01   
#define SET_FEATURE                   0x03   
#define SET_ADDRESS                   0x05
#define GET_DESCRIPTOR                0x06
#define SET_DESCRIPTOR                0x07
#define GET_CONFIGURATION             0x08
#define SET_CONFIGURATION             0x09
#define GET_INTERFACE                 0x0A
#define SET_INTERFACE                 0x0B
#define SYNCH_FRAME                   0x0C
                                      
//--- Deskriptortypen                 
#define DEVICE                        0x01
#define CONFIGURATION                 0x02
#define STRING                        0x03
#define INTERFACE                     0x04
#define ENDPOINT                      0x05

//---fr usb_get_status()
#define REQUEST_DEVICE_STATUS         0x80
#define REQUEST_INTERFACE_STATUS      0x81
#define REQUEST_ENDPOINT_STATUS       0x82   

//---fr usb_set_feature() und usb_get_feature()
#define ZERO_TYPE                     0x00
#define INTERFACE_TYPE                0x01
#define ENDPOINT_TYPE                 0x02 

//--- USB Befehle
#define Usb_select_ep(e)              (UEPNUM = e)
#define Usb_setup_received()          (UEPSTAX & MSK_RXSETUP)
#define Usb_bulk_received()           (UEPSTAX & MSK_RXOUTB0B1)
#define Usb_enable_ep_int(e)          (UEPIEN |= (0x01 << e))
#define Usb_disable_ep_int(e)         (UEPIEN &= ~(0x01 << e))
#define Usb_read_byte()               (UEPDATX)
#define Usb_write_byte(x)             (UEPDATX = x)
#define Usb_clear_rx_setup()          (UEPSTAX &= ~MSK_RXSETUP)
#define Usb_set_DIR()                 (UEPSTAX |= MSK_DIR)
#define Usb_clear_DIR()               (UEPSTAX &= ~MSK_DIR)
#define Usb_clear_rx()                (UEPSTAX &= ~MSK_RXOUT)
#define Usb_set_stall_request()       (UEPSTAX |= MSK_STALLRQ)
#define Usb_clear_stalled()           (UEPSTAX &= ~MSK_STALLED)
#define Usb_clear_stall_request()     (UEPSTAX &= ~MSK_STALLRQ)
#define Usb_stall_sent()              (UEPSTAX & MSK_STALLED)
#define Usb_set_DIR()                 (UEPSTAX |= MSK_DIR)
#define Usb_set_tx_ready()            (UEPSTAX |= MSK_TXRDY)
#define Usb_rx_complete()             (UEPSTAX & MSK_RXOUTB0)
#define Usb_tx_complete()             (UEPSTAX & MSK_TXCMPL)
#define Usb_clear_tx_ready()          (UEPSTAX &= ~MSK_TXRDY)
#define Usb_clear_tx_complete()       (UEPSTAX &= ~MSK_TXCMPL)
#define Usb_set_FADDEN()              (USBCON |= MSK_FADDEN)
#define Usb_configure_ep_type(x)      (UEPCONX = x) 
#define Usb_endpoint_interrupt()      (UEPINT != 0 )
#define Usb_suspend()                 (USBINT & MSK_SPINT)
#define Usb_resume()                  (USBINT & MSK_WUPCPU)
#define Usb_reset()                   (USBINT & MSK_EORINT)
#define Usb_sof()                     (USBINT & MSK_SOFINT)  
#define Usb_clear_reset()             (USBINT &= ~MSK_EORINT)
#define Usb_clear_resume()            (USBINT &= ~MSK_WUPCPU)
#define Usb_clear_sof()               (USBINT &= ~MSK_SOFINT)
#define Usb_clear_suspend()           (USBINT &= ~MSK_SPINT)


//*****************************************************************************
//--- USB DEVICE DESCRIPTOR
//*****************************************************************************
code t_usb_device_descriptor usb_device_descriptor =
{
  18,                                                   // bLength (Deskriptorgre) = 18
  1,                                                    // bDescriptorType (01h=Device Descriptor)
  0x0110,                                               // BcdUSB (USB-Version 1.1=0x1001)
  0,                                                    // bDeviceClass (0=Klasse wird in spterem Descriptor definiert)
  0,                                                    // bDeviceSubClass    
  0,                                                    // bDeviceProtocol                                        
  EP_CONTROL_LENGTH,                                    // bMaxPacketSize0 (Gre des EP0-FIFO)
  0x03EB,                                               // IdVendor (0xEB03=Atmel)
  0x2FFD,                                               // IdProduct (Atmel HID keyboard Bsp=0x0320)
  0x0001,                                               // bcdDevice (Versionsnummer, Atmel Bsp=0x0001)
  MAN_INDEX,                                            // iManufacturer (String-Index fr Hersteller)      
  PROD_INDEX,                                           // iProduct (String-Index fr Produkt)    
  SN_INDEX,                                             // iSerialNumber (String-Index fr Seriennr)          
  1                                                     // bNumConfigurations (Anzahl mgl. Konfigurationen)
};

//*****************************************************************************
//--- USB CONFIGURATION DESCRIPTOR
//*****************************************************************************
code t_usb_configuration_descriptor cfg =
{
  9,                                                    // bLength (Deskriptorgre)
  2,                                                    // bDescriptorType (0x02=Configuration Descriptor)                       
  0x0020,                                               // wTotalLength (Lnge aller Deskriptoren dieser
                                                        // Config) (32 Bytes: cfg=9,ifc=9, ep4out=7, ep5in=7)
  1,                                                    // bNumInterfaces (Anzahl Interfaces)
  1,                                                    // bConfigurationValue (Nr dieser Config)
  0,                                                    // iConfiguration (String-Index)
  0x80,                                                 // bmAttibutes (Attribute der Config, 0x80=BusPowered)
  50                                                    // MaxPower (Stromaufnahme in 2mA Schritten, 50=100mA)
};


//*****************************************************************************
//--- USB INTERFACE DESCRIPTOR
//*****************************************************************************
code t_usb_interface_descriptor ifc =
{
  9,                                                    // bLength (Deskriptorgre)
  4,                                                    // bDescriptorType (0x04=Interface Descriptor)    
  0,                                                    // bInterfaceNumber (Interface Nr)   
  0,                                                    // bAlternateSetting
  2,                                                    // bNumEndpoints (Anzahl Endpoints)
  0x00,                                                 // bInterfaceClass (0x03 = HID, 0x00 = keine)
  0,                                                    // bInterfaceSubClass 
  0,                                                    // bInterfaceProtocol 
  0                                                     // iInterface (String-Index)        
};


//*****************************************************************************
//--- USB ENDPOINT 4 (OUT) DESCRIPTOR [Host -> Gert]
//*****************************************************************************
code t_usb_endpoint_descriptor ep4out =
{
  0x07,                                                 // bLength (Deskriptorgre)
  0x05,                                                 // bDescriptorType (0x05=Endpoint Descriptor)    
  0x04,                                                 // bEndpointAddress (EP-Nummer und Richtung)
  0x02,                                                 // bmAttributes (Transfertyp=BULK)
  0x0040,                                               // wMaxPacketSize (max. Paketgre des EP)
  0x00                                                  // bInterval (Abfrageintervall, bei BULK=0)
};
     
//*****************************************************************************
//--- USB ENDPOINT 5 (IN) DESCRIPTOR [Gert -> Host]
//*****************************************************************************
code t_usb_endpoint_descriptor ep5in =
{
  0x07,                                                 // bLength (Deskriptorgre)                 
  0x05,                                                 // bDescriptorType (0x05=Endpoint Descriptor)
  0x85,                                                 // bEndpointAddress (EP-Nummer und Richtung) 
  0x02,                                                 // bmAttributes (Transfertyp=BULK)           
  0x0040,                                               // wMaxPacketSize (max. Paketgre des EP)   
  0x00                                                  // bInterval (Abfrageintervall, bei BULK=0)  
};

                      
//*****************************************************************************
//--- USB LANGUAGE DESCRIPTOR
//*****************************************************************************
code t_usb_language_descriptor usb_language =
{ 
 sizeof(usb_language),                                  // bLength (Deskriptorgre)
 0x03,                                                  // bDescriptorType (0x03=String Descriptor)
 LANGUAGE_ID                                            // wlangid (Sprach-ID)
};
  

//*****************************************************************************
//--- USB MANUFACTURER STRING DESCRIPTOR
//*****************************************************************************
code t_usb_manufacturer usb_manufacturer =
{                                                       
 sizeof(usb_manufacturer),                              // bLength (Deskriptorgre)
 0x03,                                                  // bDescriptorType (0x03=String Descriptor)
 USB_MANUFACTURER_NAME                                  // wstring (Unicode-String)
};
   
   
//*****************************************************************************
//--- USB PRODUCT STRING DESCRIPTOR
//*****************************************************************************
code t_usb_product usb_product =
{ 
 sizeof(usb_product),                                   // bLength (Deskriptorgre)
 0x03,                                                  // bDescriptorType (0x03=String Descriptor)
 USB_PRODUCT_NAME                                       // wstring (Unicode-String)
};
    
    
//*****************************************************************************
//--- USB SERIAL NUMBER STRING DESCRIPTOR
//*****************************************************************************
code t_usb_serial_number usb_serial_number =
{ 
 sizeof(usb_serial_number),                             // bLength (Deskriptorgre)
 0x03,                                                  // bDescriptorType (0x03=String Descriptor)
 USB_SERIAL_NUMBER                                      // wstring (Unicode-String)
};



//**************************************************************************************************************************
//**************************************************************************************************************************
//**************************************************************************************************************************

          

//*****************************************************************************
// Funktion:    sendet STALL an den Host
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_send_stall(void)
{
 Usb_clear_rx_setup();
 Usb_set_stall_request();
 while (!(Usb_stall_sent()));
 Usb_clear_stall_request();
 Usb_clear_stalled();   
}      
                 

          
//*****************************************************************************
// Funktion:    Sendet ein ZeroLengthPacket (ZLP) an den Host 
//              (fr Status-Phase bentigt)
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_send_zlp(void)
{
 UEPNUM = 0;
 Usb_set_DIR();                             // Richtung auf OUT setzen
 Usb_set_tx_ready();                        // set TXRDY, bertragung starten
 while (!(Usb_tx_complete()));              // warte bis TXCMPL
 Usb_clear_tx_complete();                   // TXCMPL lschen
}



//*****************************************************************************
// Funktion:    Resetet einen USB-Endpoint im Controller
//-----------------------------------------------------------------------------
// Parameter:   ep_num    - Nummer des Endpoints, der resetet wird
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_reset_endpoint(Uchar ep_num)
{
  UEPRST = 0x01 << ep_num ;                 // Entspr. Bit fr Endpoint setzen, FIFO Reset
  UEPRST = 0x00;                            // clearen beendet Reset
}



//*****************************************************************************
// Funktion:    Sendet ein Control-Paket via Endpoint0
//-----------------------------------------------------------------------------
// Parameter:   tbuf        - Zeiger auf Puffer mit Datenbytes, die gesendet
//                            werden sollen
//              data_length - Anzahl Bytes, die gesendet werden sollen
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
Uchar* usb_send_ep0_packet (Uchar* tbuf, Uchar data_length)
{
 Uchar i;

 Usb_select_ep(0);                              // Endpoint 0 auswhlen (Control-EP)
 for (i = data_length; i != 0 ; i--, tbuf++)    // alle Datenbytes durchlaufen
 { 
      Usb_write_byte(*tbuf);                    // und senden
 }
 Usb_set_tx_ready();                            // Setzen des TXRDY-Bits startet bertragung
 return tbuf;
}   

  
  
//*****************************************************************************
// Funktion:    liest und bearbeitet GET_INTERFACE REQUEST
//              sendet STALL zum Host zurck
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_get_interface (void)
{
  Usb_clear_rx_setup();                   // Setup-Request bearbeitet
  Usb_set_DIR();                          // Richtung auf Senden setzen
  usb_send_stall();                       // STALL senden
  Usb_clear_DIR();                        // Richtung wieder clearen
}                       

   
   
//*****************************************************************************
// Funktion:    liest und bearbeitet SET_CONFIGURATION REQUEST
//              es werden nur die Konfigurationen 0 (=unconfigured) und 1 
//              untersttzt, bei anderen Anfragen -> STALL
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_set_configuration (void)
{
 Uchar num;
 num = g_Request.wValueL;                 // Config-Nr. auslesen
 Usb_clear_DIR();                         // Richtung clearen
 Usb_clear_rx_setup();                    // Setup-Request bearbeitet

 if (num <= 1)                            // untersttze Config-Nr.
 {
    g_ConfigurationNo = num;              // aktuelle Config-Nr. global abspeichern
 }
 else
 {   
    usb_send_stall();                     // andernfalls STALL senden
    return;
 }

 Usb_set_tx_ready();                      // sende ZLP der STATUS-Phase
 while (!Usb_tx_complete());              // Warte, bis Senden abgeschlossen
 Usb_clear_tx_complete();                 // Senden abgeschlossen
 
 // Endpoint 4 fr Bulk-Out Mode neu konfigurieren  
 UEPNUM  = 4;                             // Endpoint EP4 auswhlen
 UEPCONX = 0x82;                          // EP4 aktivieren (UEPCONX=82h fr Bulk-Out Mode)
 usb_reset_endpoint(4);                   // Reset EP4 FIFO

  // Endpoint 5 fr Bulk-In Mode Transfer initialisieren
 UEPNUM  = 5;                             // Endpoint EP5 auswhlen
 UEPCONX = 0x86;                          // EP5 aktivieren (UEPCONX=80h fr Bulk-In Mode)
 usb_reset_endpoint(5);                   // Reset EP5 FIFO

 UEPIEN |= MSK_EP4INTE;                   // Interrupt fr EP4 aktivieren
}
       
         
//*****************************************************************************
// Funktion:    liest und bearbeitet GET_CONFIGURATION REQUEST 
//              schickt die aktuelle Konfigurationsnr. an den Host 
//              0 = unconfigured (->Adresszustand)
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_get_configuration (void)
{
 Usb_clear_rx_setup();                    // Setup-Request bearbeitet
 Usb_set_DIR();                           // Richtung auf SENDEN setzen
 Usb_write_byte(g_ConfigurationNo);       // aktuelle Konfigurationsnr. senden
 Usb_set_tx_ready();                      // und Paket abschicken
 while (!(Usb_tx_complete()));            // Warten, bis Paket gesendet
 Usb_clear_tx_complete();                 // Senden erfolgreich!
 while (!(Usb_rx_complete()));            // Besttigung abwarten
 Usb_clear_rx();                          // Besttigung ist angekommen
 Usb_clear_DIR();                         // Richtung wieder clearen
}      
         

        

//*****************************************************************************
// Funktion:    liest und bearbeitet SET_FEATURE REQUEST
//              Es werden nur Requests an Endpoint 4 untersttzt
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_set_feature (void)
{                                                  

 // wenn Request nicht an Endpoint gerichtet ist -> STALL senden
 if ( (g_Request.bmRequestType == ZERO_TYPE) || (g_Request.bmRequestType == INTERFACE_TYPE) )
    usb_send_stall();

 if (g_Request.bmRequestType == ENDPOINT_TYPE)    // Anfrage an Endpoint
 {
    Uchar num = g_Request.wIndexL & MSK_EP_DIR;    // Dir-Bit entfernen, um Nummer zu extrahieren
    switch (num)
    {
     case 0x04:   Usb_select_ep(0x04);                        // Bulk-Endpoint 4 auswhlen
                  Usb_set_stall_request();                    // STALL setzen
                  Usb_select_ep(0x00);                        // Control-Endpoint 0 whlen
                  g_Endpoint4Status = 0x01;                   // Status fr Endpoint 4 globale merken
                  Usb_clear_rx_setup();                       // Setup-Anfrage bearbeitet
                  usb_send_zlp(); 
                  break;
      
     case 0x05:   Usb_select_ep(0x05);                        // Bulk-Endpoint 4 auswhlen
                  Usb_set_stall_request();                    // STALL setzen
                  Usb_select_ep(0x00);                        // Control-Endpoint 0 whlen
                  g_Endpoint5Status = 0x01;                    // Status fr Endpoint 4 globale merken
                  Usb_clear_rx_setup();                       // Setup-Anfrage bearbeitet
                  usb_send_zlp(); 
                  break;
      
     default:     usb_send_stall();                           // andere Endpoints nicht untersttzt!
                  break;
      
    }
    
                              
    
 }  
}


//*****************************************************************************
// Funktion:    liest und bearbeitet CLEAR_FEATURE REQUEST
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-03, Andreas Schibilla
//*****************************************************************************
void usb_clear_feature (void)
{
 // wenn Request nicht an Endpoint gerichtet ist -> STALL senden
 if ( (g_Request.bmRequestType == ZERO_TYPE) || (g_Request.bmRequestType == INTERFACE_TYPE))
    usb_send_stall();

 if (g_Request.bmRequestType == ENDPOINT_TYPE)
 {           
     
    if (g_Request.wValueL == 0x00)                    // Feature_Selector = ENDPOINT_HALT (0x00)
    {
     
        Uchar num = g_Request.wIndexL & MSK_EP_DIR;   // Dir-Bit entfernen, um Nummer zu extrahieren
        switch(num)                                   // Welche Endpoint Nummer ist gefragt?
        {
            case 0x00:  Usb_clear_rx_setup();         // Setup-Request bearbeitet
                        usb_send_zlp();               // Antwort ZLP schicken
                        break;
                        
            case 0x04:  Usb_select_ep(0x04);          // Bulk-Endpoint 4 auswhlen
                        Usb_clear_stall_request();    // STALL clearen
                        usb_reset_endpoint(0x04);     // Endpoint 4 zurcksetzen
                        Usb_select_ep(0x00);          // Control-Endpoint 0 auswhlen
                        g_Endpoint4Status = 0x00;      // Status fr Endpoint 4 globale merken 
                        Usb_clear_rx_setup();         // Setup-Anfrage bearbeitet
                        usb_send_zlp();               // Antwort ZLP schicken
                        break; 
                        
            case 0x05:  Usb_select_ep(0x05);          // Bulk-Endpoint 4 auswhlen
                        Usb_clear_stall_request();    // STALL clearen
                        usb_reset_endpoint(0x05);     // Endpoint 4 zurcksetzen
                        Usb_select_ep(0x00);          // Control-Endpoint 0 auswhlen
                        g_Endpoint5Status = 0x00;     // Status fr Endpoint 4 globale merken 
                        Usb_clear_rx_setup();         // Setup-Anfrage bearbeitet
                        usb_send_zlp();               // Antwort ZLP schicken
                        break;
                        
            default:    usb_send_stall();             // andere Endpoints nicht untersttzt!
                        break;
          break;       
        }
    }
 }
}



//*****************************************************************************
// Funktion:    liest und bearbeitet GET_STATUS REQUEST, gibt die Merkmale
//              des Gerts, Schnittstelle oder Endpoints zurck
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-02, Andreas Schibilla
//*****************************************************************************
void usb_get_status (void)
{
 Usb_clear_rx_setup();                                            // Endpoint FIFO wieder freigeben
 Usb_set_DIR();                                                   // Richtung auf Schreiben setzen
  
 switch(g_Request.bmRequestType)                                  // wohin soll das Paket gehen?
 {
    case REQUEST_DEVICE_STATUS:     Usb_write_byte(0x00);         // nicht SelfPowered, kein RemoteWakeup
                                    break;
    
    case REQUEST_INTERFACE_STATUS:  Usb_write_byte(0x00);         // Reserved      
                                    break;
    
    case REQUEST_ENDPOINT_STATUS:   {
                                     Uchar num = g_Request.wIndexL & MSK_EP_DIR;    // Welche Endpoint-Nummer ist gefragt?
                                     switch(num)
                                     {
                                        case 0x04:  Usb_write_byte(g_Endpoint4Status);
                                                    break;
                                                    
                                        case 0x05:  Usb_write_byte(g_Endpoint5Status);
                                                    break;
                                                    
                                        default:    Usb_write_byte(0x00);
                                                    break;
                                                    //usb_send_stall();     // Ungltiger Endpoint
                                                    //return;
                                     }
                                     break;
                                    }
    default:                        usb_send_stall();                        // nicht untersttzt!
                                    return;
 }
 Usb_write_byte(0x00);                                            // 2. Datenbyte senden, Reserved=0x00 
                                                                  
 Usb_set_tx_ready();                                              // FIFO versenden durch TXRDY bit
 while ((!(Usb_tx_complete())) || (Usb_setup_received()));        // Warten bis Versand fertig
 Usb_clear_tx_complete();                                         // TXCMPL bit lschen
 while ((!(Usb_rx_complete())) || (Usb_setup_received()));        // Warten bis alles empfangen
 Usb_clear_rx();                                                  
 Usb_clear_DIR();                                                 // Bereit fr nchstes Paket
}



//*****************************************************************************
// Funktion:    liest und bearbeitet GET_DESCRIPTOR REQUEST
//              Die einzelnen Enumeration-Descriptoren werden ausgewertet 
//              und zurckgeschickt!
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-05-26, Andreas Schibilla
//*****************************************************************************
void usb_get_descriptor (void)
{
 Uchar   data_to_transfer;
 Uint16  wLength;
 Uchar   descriptor_type;
 Uchar   string_type;
 
 g_zlp = FALSE;                               // kein ZLP
 string_type      = g_Request.wValueL;        // read LSB of wValue 
 descriptor_type  = g_Request.wValueH;        // read MSB of wValue 
 wLength          = g_Request.wLengthL;       // wLength bekommt HIGH- und LOW-Anteil
 wLength         |= g_Request.wLengthH << 8;
 
 //--- je nach Descriptortyp unterscheiden...
 switch (descriptor_type)
 {
    case DEVICE:
    {
          data_to_transfer = sizeof (t_usb_device_descriptor);    // Gre merken
          g_pbuffer = &(usb_device_descriptor.bLength);           // Zeiger auf Anfang von Dev.Descr. 
          break;
    }
    case CONFIGURATION:
    {
          data_to_transfer = sizeof(cfg)+sizeof(ifc)+
                             sizeof(ep4out)+sizeof(ep5in);        // Gre merken
          g_pbuffer = &(cfg.bLength);                             // Zeiger auf Anfang von CFG-Descr.
          break;
    }   
    case INTERFACE:
    {
          data_to_transfer = sizeof(ifc);                         // Gre merken
          g_pbuffer = &(ifc.bLength);                             // Zeiger auf Anfang von IFC-Descr.
          break;
    } 
    case STRING:
    {
          switch (string_type)
          {
                case 0x00:    // Language
                {
                      data_to_transfer = sizeof (usb_language);
                      g_pbuffer = &(usb_language.bLength); 
                      break;
                }
                case MAN_INDEX:
                {
                      data_to_transfer = sizeof (usb_manufacturer);
                      g_pbuffer = &(usb_manufacturer.bLength);
                      break;
                }
                case PROD_INDEX:
                {
                      data_to_transfer = sizeof (usb_product);
                      g_pbuffer = &(usb_product.bLength);
                      break;
                }
                case SN_INDEX:
                {
                      data_to_transfer = sizeof (usb_serial_number);
                      g_pbuffer = &(usb_serial_number.bLength);
                      break;
                }
                default:
                {
                      Usb_clear_rx_setup();
                      usb_send_stall();
                      Usb_clear_DIR();
                      return;
                }
          }
          break;
    }      

    default:
    {
          Usb_clear_rx_setup();                                   // Setup-Request bearbeitet
          usb_send_stall();                                       // STALL senden
          return;
    }
 }
 
 //--- je nach Lngenangabe (wLength) Anz. der zu bertr. Bytes einstellen
 if (wLength > data_to_transfer)                                  // wenn Descriptor kleiner als wLength
 {      
    g_zlp = ((data_to_transfer % EP_CONTROL_LENGTH) == 0);        // wenn Descr. gerades Vielfaches der 
                                                                  // max. Paketlnge, soll ZLP folgen
 }                                                                
 else                                                             // wenn Descriptor grer als wLength
 {
    data_to_transfer = (Uchar)wLength;                            // nur angeforderte Anz. Bytes zurcksenden
 }

 
 Usb_clear_rx_setup() ;                                           // Setup Paket bearbeitet
 Usb_set_DIR();                                                   // Richtung auf senden

 //--- Descriptor senden (ggf. mehrere Pakete)
 while (data_to_transfer > EP_CONTROL_LENGTH)                     // Descr. grer als max. Paketlnge EP0
 {
    g_pbuffer = usb_send_ep0_packet(g_pbuffer,EP_CONTROL_LENGTH); // max. Paket senden
    data_to_transfer -= EP_CONTROL_LENGTH;                        // und restl. Bytes merken

    while ((!(Usb_rx_complete())) && (!(Usb_tx_complete())));     // warten bis Senden fertig
    Usb_clear_tx_complete();                                      // gesendet!
    if ((Usb_rx_complete()))                                      // wenn Host cancelt
    {
      Usb_clear_tx_ready();                                       // bertragung abbrechen
      Usb_clear_rx();
      return;
    }
 }                       
 
 //-- den fehlenden Rest senden (letztes Datenpaket)
 g_pbuffer = usb_send_ep0_packet(g_pbuffer, data_to_transfer);    // letztes Paket senden
 data_to_transfer = 0;
 while ((!(Usb_rx_complete())) && (!(Usb_tx_complete())));        // warten bis Senden fertig
 Usb_clear_tx_complete();                                         // gesendet!
 if ((Usb_rx_complete()))                                         // wenn Host cancelt
 {
    Usb_clear_tx_ready();                                         // bertragung abbrechen
    Usb_clear_rx();
    return;
 }
 
 //--- ggf. noch ein ZLP senden    
 if (g_zlp)
 {
    usb_send_ep0_packet(g_pbuffer, 0);
    while ((!(Usb_rx_complete())) && (!(Usb_tx_complete())));     // warten bis Senden fertig
    Usb_clear_tx_complete();                                      // gesendet!
    if ((Usb_rx_complete()))                                      // wenn Host cancelt
    {
      Usb_clear_tx_ready();                                       // bertragung abbrechen
      Usb_clear_rx();
      return;
    }

 }
 
 //--- Status Handshake
 while ((!(Usb_rx_complete())) && (!(Usb_setup_received())));     // Warte, bis Besttigung empfangen
 if (Usb_setup_received())
 {
    return;
 }
 if (Usb_rx_complete())
 {
    Usb_clear_DIR();                                              
    Usb_clear_rx();
 } 
}         



//*****************************************************************************
// Funktion:    liest und bearbeitet SET_ADRESS REQUEST, die neue Adresse wird
//              aus den Paketdaten extrahiert und im Controller gesetzt
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-05-26, Andreas Schibilla
//*****************************************************************************
void usb_set_address (void)
{
 Usb_clear_rx_setup();                      // EP FIFO fr neuen Empfang clearen
 Usb_set_FADDEN();                          // Nur noch auf die neue Adresse reagieren
 usb_send_zlp();                            // ZeroLengthPacket (ZLP) senden
 USBADDR = (0x80 | g_Request.wValueL);      // neue USB-Adresse (steht in wValueL) setzen
} 


//*****************************************************************************
// Funktion:    Liest ein USB Setup-Packet ein (8 Bytes) und speichert es
//              in der globalen Variablen g_Request
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-05-26, Andreas Schibilla
//*****************************************************************************
void usb_read_setup_data(void)
{
 g_Request.bmRequestType  = Usb_read_byte();  
 g_Request.bRequest       = Usb_read_byte();  
 g_Request.wValueL        = Usb_read_byte();  
 g_Request.wValueH        = Usb_read_byte();  
 g_Request.wIndexL        = Usb_read_byte();  
 g_Request.wIndexH        = Usb_read_byte();  
 g_Request.wLengthL       = Usb_read_byte();  
 g_Request.wLengthH       = Usb_read_byte();  
}



//*****************************************************************************
// Funktion:    USB Enumeration / Setup Stage
//              Analysiert ein Setup-Request-Paket und wertet es aus
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-05-26, Andreas Schibilla
//*****************************************************************************
void usb_enumeration_process(void)
{ 
  
 usb_read_setup_data();                                       // Setup-Request-Paket einlesen

 switch (g_Request.bRequest)                                  // Was fr ein Request ist gefragt?
 {     
      case GET_STATUS:        usb_get_status();
                              break;
      case CLEAR_FEATURE:     usb_clear_feature();
                              break;
      case SET_FEATURE:       usb_set_feature();        
                              break;
      case SET_ADDRESS:       usb_set_address();
                              break;
      case GET_DESCRIPTOR:    usb_get_descriptor();
                              break;
      case GET_CONFIGURATION: usb_get_configuration();
                              break;
      case SET_CONFIGURATION: usb_set_configuration();
                              break;
      case GET_INTERFACE:     usb_get_interface();      
                              break;
      case SET_DESCRIPTOR:
      case SET_INTERFACE:
      case SYNCH_FRAME:
      default:                Usb_clear_rx_setup();           // Setup-Request bearbeitet
                              usb_send_stall();               // STALL senden
                              break;
    }
 Usb_clear_DIR();                                             // Richtung auf IN setzen
}
  
  

//*****************************************************************************
// Funktion:    Sendet ein USB-BULK-Paket
//-----------------------------------------------------------------------------
// Parameter:   sig - Signatur-Byte (Kennzeichnung, sollte 'A' sein)
//              cmd - Befehlsbyte
//              p   - Zeiger auf eigentliche Paketdaten
//              num - Anzahl Paketdaten
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-05-20, Andreas Schibilla
//*****************************************************************************
void usb_send_data(char sig, char cmd, char *p, unsigned char num)
{   
 unsigned char i;
 if (num <= 0)
    return;
    
 UEPNUM = 5;            // EP5 auswhlen
 UEPDATX = sig;         // Signatur senden
 UEPDATX = cmd;         // Befehl oder Errorcode senden
 for (i=0; i<num; ++i)  // Datenbytes senden
 {
    UEPDATX = p[i];
 }
 Usb_set_tx_ready();
 while (!Usb_tx_complete());
 Usb_clear_tx_complete();   
}

    
    
//*****************************************************************************
// Funktion:    Verarbeitet empfangene Bulk-Pakete von Endpoint 4
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-01, Andreas Schibilla
//*****************************************************************************
void usb_bulk_process(void)
{    
 char num;
 char i;
 
 num = UBYCTLX;   // Anzahl lesen
 for (i=0; i<num; ++i)
 {
    g_buf[i] = UEPDATX;
 }

 UEPSTAX &= !0x42;
 EP4INT   = 0;
           
 // empfangenes Paket auswerten, Befehl ausfhren und Antwort entspr. zurckschicken
 usb_eval(g_buf, num);
}
    
    

//*****************************************************************************
// Funktion:    USB Interrupt Service Routine; Verarbeitet alle USB-Interrupt-
//              Anfragen: 
//              (1) Setup-Pakete empfangen  -> Enumerationproze ausfhren
//              (2) Bulk-Pakete empfangen   -> Nutzdaten verarbeiten
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-05-24, Andreas Schibilla
//*****************************************************************************
void usb_interrupt_process(void) interrupt IRQ_USB
{
 if (Usb_suspend())
 {
        if (Usb_resume())
        {
          Usb_clear_suspend();
          Usb_clear_resume();
          Usb_clear_sof();
        }
 }
 else
 {        
        if (Usb_sof())
        {
          Usb_clear_sof();
        }

        if (Usb_endpoint_interrupt())
        {
             //--- Bulk-Transfer: Nutzdaten
             if (EP4INT)
             { 
                 Usb_select_ep(4);                         // Endpoint 4 auswhlen
                 if (Usb_bulk_received())                  // wenn Bulk-Paket empfangen
                 {                                                                                   
                     LED_BUSY = 0;                         // BUSY anzeigen
                     Usb_disable_ep_int(4);                // Interrupt sperren, bis Paket bearbeitet
                     Usb_select_ep(4);                     // Endpoint 4 auswhlen
                     usb_bulk_process();                   // Empfangenes Bulk-Paket verarbeiten
                     Usb_enable_ep_int(4);                 // Interrupt wieder freigeben
                     LED_BUSY = 1;                         // READY anzeigen
                 }
             }
              //--- Control-Transfer: Setup
             if (EP0INT)  
             { 
                 Usb_select_ep(0);                         // Endpoint 0 auswhlen
                 if (Usb_setup_received())                 // wenn Setup-Paket empfangen
                 { 
                     LED_BUSY = 0;                         // BUSY anzeigen
                     Usb_disable_ep_int(0);                // Interrupt sperren, bis Paket bearbeitet
                     Usb_select_ep(0);                     // Endpoint 0 auswhlen
                     usb_enumeration_process();            // Paket verarbeiten, Enumeration abarbeiten
                     Usb_enable_ep_int(0);                 // Interrupt wieder freigeben
                     LED_BUSY = 1;                         // READY anzeigen
                 } 
             }
         
        } // end USB Endpoint Interrupt
 } // else Usb_suspend()       
}



//*****************************************************************************
// Funktion:    Initialisiert alle fr USB notwendigen Register, aktiviert den
//              USB Controller, stellt die Endpoints 0,4,5 ein und schaltet
//              die USB Interrupts frei!
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-05-24, Andreas Schibilla
//*****************************************************************************
void usb_init(void)
{
 
 USBCON |= MSK_USBE;        // USB Controller aktivieren (USBE Bit in USBCON setzen)
 UEPIEN |= MSK_EP4INTE;     // Interrupt fr Endpoint 4 freischalten
 UEPIEN |= MSK_EP0INTE;     // Interrupt fr Endpoint 0 freischalten


 //--- Alle Endpoints einstellen (S. 120 im Datenblatt)
 // Endpoint 0 fr Control Mode Transfer initialisieren
 UEPNUM  = 0;               // Endpoint EP0 auswhlen
 UEPCONX = 0x80;            // EP0 aktivieren (UEPCONX=80h fr Control Mode)
 usb_reset_endpoint(0);     // Reset EP0 FIFO
 
 // Endpoint 4 fr Bulk-Out Mode Transfer initialisieren
 UEPNUM  = 4;               // Endpoint EP4 auswhlen
 UEPCONX = 0x82;            // EP4 aktivieren (UEPCONX=82h fr Bulk-Out Mode)
 usb_reset_endpoint(4);     // Reset EP4 FIFO

  // Endpoint 5 fr Bulk-In Mode Transfer initialisieren
 UEPNUM  = 5;               // Endpoint EP5 auswhlen
 UEPCONX = 0x86;            // EP5 aktivieren (UEPCONX=80h fr Bulk-In Mode)
 usb_reset_endpoint(5);     // Reset EP5 FIFO

 IEN1   |= MSK_EUSB;        // USB Interrupt freischalten
 EA      = 1;               // Alle Interrupts freischalten
}



//*****************************************************************************
// Funktion:    Initialisiert die PLL des AT89C5131 auf die Freq 48MHz, wird
//              fr USB-bertragungen bentigt!
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
// Bemerkung:   Es muss ein 16MHz Quarz in der Schaltung verwendet werden,
//              andernfalls muss der Teiler "PLLDIV" angepasst werden
//              (PLLDIV = 0x20 fr 16MHz-Quarz, siehe Datenblatt S.14)
//-----------------------------------------------------------------------------
// Datum:       2005-05-20, Andreas Schibilla
//*****************************************************************************
void usb_init_pll(void)
{
 PLLDIV = 0x20;                         // Teiler einstellen fr 16 MHz Oszillator (N+1=1, R+1=3 => PLLDIV=20h)
 PLLCON |= MSK_PLLEN;                   // PLL aktivieren (PLLEN=1)
 while (!(PLLCON & MSK_PLOCK) ) { ; }   // Warten, bis PLL eingerastet ist
}                   

